use rust_networking_examples::Network;

mod net;

use net::{DispatchHandle, Net};

/*
    main.rs

    A simple test program demonstrating sending a message between two instances of Net.
*/

/// Our main function decorated with the tokio::main attribute to setup an async runtime.
#[tokio::main]
async fn main() {
    // Create a single dispatch actor, which will handle message routing
    let dispatcher = DispatchHandle::new();

    // Create two network instances, with addresses "a" and "b"
    let mut net_a = Net::new("a", dispatcher.clone()).await;
    let mut net_b = Net::new("b", dispatcher).await;
    
    // Channel for b to receive messages 
    let mut recv_b = net_b.recv().await;

    // Send message from a to b
    net_a.send("b", "Hello world!".as_bytes().to_vec());

    // See what b receives
    while let Some((from, msg)) = recv_b.recv().await {
        println!("b received {:?} from {}", String::from_utf8(msg).unwrap(), from);
    }
}
