mod dispatch;

use rust_networking_examples::Network;
use tokio::sync::mpsc;

pub use dispatch::DispatchHandle;

/// Handle for sending and receiving on our network simulated using async channels.
#[derive(Clone)]
pub struct Net {
    /// Handle to our dispatcher, for sending messager
    dispatcher: DispatchHandle,

    /// Our address
    address: String,
}

impl Net {
    pub async fn new(address: &str, dispatcher: DispatchHandle) -> Net {
        Net { dispatcher, address: address.to_string() }
    }
}

// Here we implement our Network interface.
#[async_trait::async_trait]
impl Network for Net {
    fn send(&mut self, to: &str, message: Vec<u8>) {
        // Send message to the dispatch actor
        self.dispatcher.send_message(to, &self.address, message);
    }

    async fn recv(&mut self) -> mpsc::Receiver<(String, Vec<u8>)> {
        // Register a new sender with the dispatch actor
        self.dispatcher.register(&self.address).await
    }
}