use tokio::sync::mpsc;

/// Our Network interface which is accomplished through a language feature of Rust called 'traits'
/// 
/// Note the async_trait annotation; this adds support for defining async functions as part of your interface. The stable version of rust currently doesn't have this, so we rely on a crate as a workaround.
#[async_trait::async_trait]
pub trait Network {
    /// Send a message to the identifier specified by the 'to' field.
    fn send(&mut self, to: &str, message: Vec<u8>);

    /// Generate one or more receivers to catch inbound messages.
    async fn recv(&mut self) -> mpsc::Receiver<(String, Vec<u8>)>;
}